@@ -362,7 +362,7 @@ template <> QNavigationManagerEngine *createEngine<QNavigationManagerEngine>(QGe
 {
     if (!d_ptr->factoryV2)
         return nullptr;
-    return d_ptr->factoryV2->createNavigationManagerEngine(d_ptr->cleanedParameterMap, &(d_ptr->placeError), &(d_ptr->placeErrorString));
+    return d_ptr->factoryV2->createNavigationManagerEngine(d_ptr->cleanedParameterMap, &(d_ptr->navigationError), &(d_ptr->navigationErrorString));
 }
 
 /* Template for generating the code for each of the geocodingManager(),
@@ -381,11 +381,15 @@ Manager *QGeoServiceProviderPrivate::manager(QGeoServiceProvider::Error *_error,
         this->loadPlugin(this->parameterMap);
     }
 
-    if (!this->factory || error != QGeoServiceProvider::NoError)
+    if (!this->factory) {
+        error = this->error;
+        errorString = this->errorString;
         return 0;
+    }
 
     if (!manager) {
-        Engine *engine = createEngine<Engine>(this);
+        Engine *engine = createEngine<Engine>(this); // this sets the specific error variables directly,
+                                                     // from now on the local error, errorString refs should be set.
 
         if (engine) {
             engine->setManagerName(
@@ -534,9 +538,13 @@ QPlaceManager *QGeoServiceProvider::placeManager() const
 */
 QNavigationManager *QGeoServiceProvider::navigationManager() const
 {
-    return d_ptr->manager<QNavigationManager, QNavigationManagerEngine>(
+    QNavigationManager * mgr = d_ptr->manager<QNavigationManager, QNavigationManagerEngine>(
                &(d_ptr->navigationError), &(d_ptr->navigationErrorString),
                 &(d_ptr->navigationManager));
+    if (!mgr) {
+        qDebug() << d_ptr->navigationError << d_ptr->navigationErrorString;
+    }
+    return mgr;
 }
 
 /*!
@@ -572,6 +580,11 @@ void QGeoServiceProvider::setAllowExperimental(bool allow)
     d_ptr->loadMeta();
 }
 
+void QGeoServiceProvider::setQmlEngine(QQmlEngine *engine)
+{
+    d_ptr->qmlEngine = engine;
+}
+
 /*!
     Sets the parameters used to construct individual manager classes for
     this service provider to \a parameters.
@@ -660,7 +673,7 @@ void QGeoServiceProviderPrivate::unload()
     delete navigationManager;
     navigationManager = nullptr;
 
-    factory = factoryV2 = nullptr;
+    factory = factoryV2 = factoryV3 = nullptr;
     error = QGeoServiceProvider::NoError;
     errorString = QLatin1String("");
     metaData = QJsonObject();
@@ -690,7 +703,7 @@ void QGeoServiceProviderPrivate::filterParameterMap()
 
 void QGeoServiceProviderPrivate::loadMeta()
 {
-    factory = factoryV2 = nullptr;
+    factory = factoryV2 = factoryV3 = nullptr;
     metaData = QJsonObject();
     metaData.insert(QStringLiteral("index"), -1);
     error = QGeoServiceProvider::NotSupportedError;
@@ -731,7 +744,7 @@ void QGeoServiceProviderPrivate::loadPlugin(const QVariantMap &parameters)
     if (int(metaData.value(QStringLiteral("index")).toDouble()) < 0) {
         error = QGeoServiceProvider::NotSupportedError;
         errorString = QString(QLatin1String("The geoservices provider is not supported."));
-        factory = factoryV2 = nullptr;
+        factory = factoryV2 = factoryV3 = nullptr;
         return;
     }
 
@@ -742,11 +755,22 @@ void QGeoServiceProviderPrivate::loadPlugin(const QVariantMap &parameters)
 
     // load the actual plugin
     QObject *instance = loader()->instance(idx);
-    factoryV2 = qobject_cast<QGeoServiceProviderFactoryV2 *>(instance);
-    if (!factoryV2)
-        factory = qobject_cast<QGeoServiceProviderFactory *>(instance);
-    else
-        factory = factoryV2;
+    if (!instance) {
+        error = QGeoServiceProvider::LoaderError;
+        errorString = QLatin1String("loader()->instance(idx) failed to return an instance. Set the environment variable QT_DEBUG_PLUGINS to see more details.");
+        return;
+    }
+    factoryV3 = qobject_cast<QGeoServiceProviderFactoryV3 *>(instance);
+    if (!factoryV3) {
+        factoryV2 = qobject_cast<QGeoServiceProviderFactoryV2 *>(instance);
+        if (!factoryV2)
+            factory = qobject_cast<QGeoServiceProviderFactory *>(instance);
+        else
+            factory = factoryV2;
+    } else {
+        factory = factoryV2 = factoryV3;
+        factoryV3->setQmlEngine(qmlEngine);
+    }
 }
 
 QHash<QString, QJsonObject> QGeoServiceProviderPrivate::plugins(bool reload)
