@@ -44,11 +44,28 @@
 #include <QHash>
 #include <QDataStream>
 #include <QDebug>
+#include <QMetaType>
 #include <qnumeric.h>
 #include <qmath.h>
 
 QT_BEGIN_NAMESPACE
 
+
+struct CoordinateStreamOperators
+{
+    CoordinateStreamOperators()
+    {
+#ifndef QT_NO_DATASTREAM
+        qRegisterMetaTypeStreamOperators<QGeoCoordinate>();
+#endif
+#ifndef QT_NO_DEBUG_STREAM
+        QMetaType::registerDebugStreamOperator<QGeoCoordinate>();
+#endif
+    }
+};
+Q_GLOBAL_STATIC(CoordinateStreamOperators, initStreamOperators);
+
+
 static const double qgeocoordinate_EARTH_MEAN_RADIUS = 6371.0072;
 
 
@@ -201,6 +218,9 @@ QGeoMercatorCoordinatePrivate::~QGeoMercatorCoordinatePrivate()
 QGeoCoordinate::QGeoCoordinate()
         : d(new QGeoCoordinatePrivate)
 {
+#ifndef QT_NO_DATASTREAM
+    initStreamOperators();
+#endif
 }
 
 /*!
@@ -215,6 +235,10 @@ QGeoCoordinate::QGeoCoordinate()
 QGeoCoordinate::QGeoCoordinate(double latitude, double longitude)
         : d(new QGeoCoordinatePrivate)
 {
+#ifndef QT_NO_DATASTREAM
+    initStreamOperators();
+#endif
+
     if (QLocationUtils::isValidLat(latitude) && QLocationUtils::isValidLong(longitude)) {
         d->lat = latitude;
         d->lng = longitude;
@@ -236,6 +260,10 @@ QGeoCoordinate::QGeoCoordinate(double latitude, double longitude)
 QGeoCoordinate::QGeoCoordinate(double latitude, double longitude, double altitude)
         : d(new QGeoCoordinatePrivate)
 {
+#ifndef QT_NO_DATASTREAM
+    initStreamOperators();
+#endif
+
     if (QLocationUtils::isValidLat(latitude) && QLocationUtils::isValidLong(longitude)) {
         d->lat = latitude;
         d->lng = longitude;
